// Guild Wars Character Builder - Assists in attribute layout.
// Copyright (C) 2004  Joshua Lambert & Adam Kovalik
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


using System;
using System.Collections;
using System.Xml;
using System.Windows.Forms;
using System.Drawing;

namespace gwc
{
	
	//Individual Class for a class's data.
	class ClassStruct {
			public ClassStruct() {
				for(int i=0;i<5;i++) {
					alPrim[i]=false;
				}
			}	
			public string name; //Name of the Class.
			public ArrayList alAttributes = new ArrayList(); //Array of strings containing the Attribute names with : after.
			public ArrayList alDescs = new ArrayList(); //Array of descriptions of the attributes for tooltips.
			public bool [] alPrim = new bool[5];
			
			//Required for Hashtable support.  Simple operator overloads.
			public override bool Equals(Object e) {
				if(((ClassStruct)e).name == this.name) return true; else return false;
			}
			
			public static bool operator == (ClassStruct a, ClassStruct b) {
				if(a.name == b.name) return true; else return false;
			}
			
			public static bool operator != (ClassStruct a, ClassStruct b) {
				if(a.name == b.name) return false; else return true;
			}
			
			//Name should be unique, use that for the hashcode.
			public override int GetHashCode() {
				return name.GetHashCode();
			}
		}
	
	//Wrapper for all class data.
	class AllClasses {
		private Hashtable Classes = new Hashtable(); //Holds call individual classes.
		
		public AllClasses() {
		
		}
		
		//Load class data from an XML data file.
		public bool XMLLoad (string s) {
			string prev = "";
			int i=0;
			ClassStruct cls=null;
			
			Classes.Clear();
			
			try {
				XmlTextReader xread = new XmlTextReader(s);
				while (xread.Read()) {
					 switch (xread.NodeType) {
					 	case XmlNodeType.Element:
					 		if(xread.Name == "CLASS") {
					 			cls = new ClassStruct();
					 			i=0;
					 		} else if(xread.Name == "NAME" || xread.Name == "ANAME" || xread.Name == "DESC") {
					 			prev = xread.Name;
					 		}
					 		if (xread.HasAttributes) {
			        			if(xread.GetAttribute("PRIMARY") == "true") {
			        			   cls.alPrim[i] = true;
			        			}
					 		}
			            	break;
			 
			            case XmlNodeType.EndElement:
			            	if(xread.Name == "CLASS") {
			            		this.Classes.Add(cls.name,cls);
			            	}
			            	if(xread.Name == "ATTRIBUTE") {
			            		i++;
			            	}
			             	break;
					 	
			            case XmlNodeType.Text:
			            	if(prev == "NAME") {
			            		cls.name = xread.Value;
			            	} else if (prev == "ANAME") {
			            		cls.alAttributes.Add(xread.Value);
			            	} else if (prev == "DESC") {
			            		cls.alDescs.Add(xread.Value);
			            	}
			            	break;
					 						 		
			            default:
		             		break;
					}
				
				}
				return true;
			}
			catch {
				return false;
			}
		} //End of XMLLoad
		
		//Allow reference by string.
		public ClassStruct get(string s) {
			return (ClassStruct) this.Classes[s];
		}
		
		//Allow retrieval of specific Attribute Names and Descriptions without requiring a class call first.
		public string getAName(string s,int i) {
			try {
				return (string) ((ClassStruct)this.Classes[s]).alAttributes[i];
			}
			catch {
				return null;
			}
		}
		
		public string getDesc(string s,int i) {
			try {
				return (string) ((ClassStruct)this.Classes[s]).alDescs[i];
			}
			catch {
				return null;
			}
		}
		
		public bool isPrim(string s,int i) {
			try {
				return ((ClassStruct)this.Classes[s]).alPrim[i];
			}
			catch {
				return false;
			}
		}
		
		//Allow access to total class count.
		public int Count() {
			return this.Classes.Count;
		}
		
		//Allow access to retrieve all of the class names.
		public string[] getNames() {
			string[] s = new string[this.Classes.Count];
			this.Classes.Keys.CopyTo(s,0);
			return s;
		}
		
	
	}
	
	//Simple ControlArray classes.  Allow index[] access, and takes location data for new controls.
	class LabelArray : System.Collections.CollectionBase {
		private readonly System.Windows.Forms.Form mainForm;
		
		public LabelArray(System.Windows.Forms.Form f) {
			mainForm = f;
		}
		
		public System.Windows.Forms.Label Add(int l, int t, int w, int h) {
			System.Windows.Forms.Label nl = new System.Windows.Forms.Label();
			
			nl.Top = t;
			nl.Left = l;
			nl.Width = w;
			nl.Height = h;
			nl.Tag = this.Count;
			nl.Font = new Font("Tahoma", 12);
			this.List.Add(nl);
			mainForm.Controls.Add(nl);
			
			return nl;
		}
		
		public System.Windows.Forms.Label this [int Index] {
			get {
				return (System.Windows.Forms.Label) this.List[Index];
			}
		}
	
	}
	
	class SpinArray : System.Collections.CollectionBase {
		private readonly System.Windows.Forms.Form mainForm;
		
		public SpinArray(System.Windows.Forms.Form f) {
			mainForm = f;
		}
		
		public System.Windows.Forms.NumericUpDown Add(int l, int t, int w, int h) {
			System.Windows.Forms.NumericUpDown nl = new System.Windows.Forms.NumericUpDown();
			
			nl.Top = t;
			nl.Left = l;
			nl.Width = w;
			nl.Height = h;
			nl.Tag = this.Count;
			nl.Minimum = 0;
			nl.Maximum = 12;
			this.List.Add(nl);
			mainForm.Controls.Add(nl);
			
			return nl;
		}
		
		public System.Windows.Forms.NumericUpDown this [int Index] {
			get {
				return (System.Windows.Forms.NumericUpDown) this.List[Index];
			}
		}
		
	}
}
