// Guild Wars Character Builder - Assists in attribute layout.
// Copyright (C) 2004  Joshua Lambert & Adam Kovalik
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

using System;
using System.IO;
using System.Net;
using System.Xml;
using System.Collections;
using System.Windows.Forms;


namespace gwc
{
	/// <summary>
	/// This is the only form for the program.  It contains all of the contros.	
	/// </summary>
	public class MainForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label label;
		private System.Windows.Forms.MenuItem menuHash;
		private System.Windows.Forms.TextBox attribBox;
		private System.Windows.Forms.NumericUpDown levelBox;
		private System.Windows.Forms.MenuItem menuSave;
		private System.Windows.Forms.SaveFileDialog filesave;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.MenuItem menuMode;
		private System.Windows.Forms.MenuItem menuFree;
		private System.Windows.Forms.MenuItem menuTools;
		private System.Windows.Forms.ComboBox secBox;
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.MenuItem menuLevel;
		private System.Windows.Forms.LinkLabel linkLabel;
		private System.Windows.Forms.MenuItem menuQuit;
		private System.Windows.Forms.MenuItem menuLoad;
		private System.Windows.Forms.MenuItem menuHelp;
		private System.Windows.Forms.TextBox nameBox;
		private System.Windows.Forms.MenuItem menuUpdate;
		private System.Windows.Forms.MenuItem menuFile;
		private System.Windows.Forms.MenuItem menuAbout;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Button levelbutton;
		private System.Windows.Forms.ComboBox primBox;
		private System.Windows.Forms.OpenFileDialog fileload;
		
		AllClasses cClasses;  //A Class which holds all the data on the classes.
		LabelArray lA; //Control Array which holds the attribute labels.
		LabelArray cA; //Control Array which holds the attribute cost labels.
		SpinArray sA; //Control Array which holds the NumericUpDown controls for the attributes.
		
		ToolTip tips = new ToolTip();

		int maxAttributes; //Global variable to store the total number of available skill points.
		int currentLevel; //Global Variable which stores the current level.
		int usedAttributes; //Number of currently assigned attribute points.
		
		//Array that holds the previous values of the control array.
		private int[] previousValues = new int[] {0,0,0,0,0,0,0,0,0,0,0};
		
		//Array that holds the individual and cumulative costs for each attribute level.
		private readonly int[] cost = new int[] {0,1,3,6,10,15,21,28,37,48,61,77,97};
		private readonly int[] stand = new int[] {0,1,2,3,4,5,6,7,9,11,13,16,20};
		
		private readonly int MAX_LEVEL = 20;
		private readonly int POINTS_PER_LEVEL = 10;

		//Recalculates the number of attribute points left and updates the control.
		private void UpdateDisplay()
		{
			int i;
			this.usedAttributes=0;
			for(i=0;i<11;i++) {
				this.usedAttributes+=cost[this.previousValues[i]];
			}
			int j = this.maxAttributes - this.usedAttributes;
			this.attribBox.Text = j.ToString();
		}
		
		//Simple byte-by-byte comparison of a 16byte md5 hash.
		private bool HashCmp(byte[] b1, byte[] b2) {
			for(int i=0;i<16;i++) {
				if(b1[i] != b2[i]) return false;
			}
			return true;
		}
		
		bool Download(string s, string d) {
			try {
				FileStream fileData = new FileStream(d,FileMode.Create,FileAccess.Write);
				byte[] buf = new byte[256];
				WebRequest myReq = WebRequest.Create(s);
				WebResponse myResp = myReq.GetResponse();
				Stream recv = myResp.GetResponseStream();
				int count = recv.Read(buf,0,256);
				while (count > 0) {
					fileData.Write(buf,0,count);
					count = recv.Read(buf,0,256);
				}
				fileData.Close();
				recv.Close();
				myResp.Close();
				return true;
			}
			catch {
				return false;
			}
		}
		
		//Computes and compares the hashes for the .exe and .xml files with the current copies on the server.  Updates the .xml if needed, and notifies upon .exe update.
		private bool UpdateFiles() {
			byte[] Data;
			byte[] Data2 = new byte[16];
			bool updated=false;
			System.Security.Cryptography.MD5 hash = new System.Security.Cryptography.MD5CryptoServiceProvider();
			
			try {
				FileStream fileData = new FileStream("gwc.xml",FileMode.OpenOrCreate,FileAccess.Read);
				Data = hash.ComputeHash(fileData);
				fileData.Close();
				
				WebRequest myReq = WebRequest.Create("http://gwdb/gwc/gwc.xml.md5");
				WebResponse myResp = myReq.GetResponse();
				Stream recv = myResp.GetResponseStream();
				recv.Read(Data2,0,16);
				
				recv.Close();
				myResp.Close();
				
				
				if(!HashCmp(Data,Data2)) {
					Download("http://gwdb.org/gwc/gwc.xml","gwc.xml");
					Download("http://gwdb.org/gwc/README.TXT","README.TXT");
					updated=true;
				}
				
				fileData = new FileStream("gwc.exe",FileMode.Open,FileAccess.Read);
				Data = hash.ComputeHash(fileData);
				fileData.Close();
			
				myReq = WebRequest.Create("http://gwdb.org/gwc/gwc.exe.md5");
				myResp = myReq.GetResponse();
				recv = myResp.GetResponseStream();
				recv.Read(Data2,0,16);
				
				recv.Close();
				myResp.Close();
					
				if(!HashCmp(Data,Data2)) {
					Download("http://gwdb.org/gwc/gwc.exe","gwc.new.exe");
					Download("http://gwdb.org/gwc/README.TXT","README.TXT");
					this.Close(); //Schedule program to exit.
					System.Diagnostics.Process.Start("gwc.new.exe","update");
					updated=true;
				}
				
				if(!updated) {
					MessageBox.Show("No update necessary","Update",MessageBoxButtons.OK,MessageBoxIcon.Information);
				}
			}
			catch {
				MessageBox.Show("Error updating.  If this persists please download a new .exe","Critical Error",MessageBoxButtons.OK,MessageBoxIcon.Error);
				return false;
			}
			
			return true;
		}

		//Bounds checks the current attribute values to see if they are possible given the current level.
		private bool RecalcAttrib()
		{
			int i = 0;
			for(int j=0;j<11;j++) {
				i+=cost[(int)sA[j].Value];
			}

			if(i <= this.maxAttributes)
			{
				return true;
			}
			else
			{
				return false;
			}
		}
	
		//Initializes the control arrays to with the correct values for all 9 attributes and associated controls.  Also sets default values for the tooltips control.
		void InitControls() {
			lA = new LabelArray(this);
			int i;
			
			for(i=0;i<6;i++) {
				lA.Add(8,136+(i*40),132,23);
			}
			for(i=0;i<5;i++) {
				lA.Add(288,136+(i*40),132,23);
			}
			
			cA = new LabelArray(this);
			
			for(i=0;i<6;i++) {
				cA.Add(136,136+(i*40),24,24);
				cA[(i*2)].TextAlign = System.Drawing.ContentAlignment.MiddleRight;
				cA[(i*2)].Text = "--";
				cA.Add(208,136+(i*40),24,24);
				cA[(i*2)+1].TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
				cA[(i*2)+1].Text = "1";
			}
			
			for(i=6;i<11;i++) {
				cA.Add(416,136+((i-6)*40),24,24);
				cA[(i*2)].TextAlign = System.Drawing.ContentAlignment.MiddleRight;
				cA[(i*2)].Text = "--";
				cA.Add(488,136+((i-6)*40),24,24);
				cA[(i*2)+1].TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
				cA[(i*2)+1].Text = "1";
			}
			
			sA = new SpinArray(this);
			
			for(i=0;i<6;i++) {
				sA.Add(160,136+(i*40),48,21);
				sA[i].TextAlign = HorizontalAlignment.Center;
				sA[i].ValueChanged += new System.EventHandler(SpinValueChanged);
				sA[i].LostFocus += new System.EventHandler(SpinValueChanged);
				sA[i].TabIndex = 5 + i;
			}
			
			for(i=6;i<11;i++) {
				sA.Add(440,136+((i-6)*40),48,21);
				sA[i].TextAlign = HorizontalAlignment.Center;
				sA[i].LostFocus += new System.EventHandler(SpinValueChanged);
				sA[i].ValueChanged += new System.EventHandler(SpinValueChanged);
				sA[i].TabIndex = 6 + i;
			}
			
			secBox.Items.Clear();
			secBox.Items.AddRange(cClasses.getNames());
			primBox.Items.Clear();
			primBox.Items.AddRange(cClasses.getNames());
			
			tips.AutoPopDelay = 5000;
  			tips.InitialDelay = 1000;
   			tips.ReshowDelay = 500;
   			tips.ShowAlways = true;
			
			this.levelBox.Maximum = new System.Decimal(new int[] {
						MAX_LEVEL,
						0,
						0,
						0}); 

		}
		
		//Check to see if secondary class is available, if so make it visible, if not, don't.
		void CheckLevel() {
			int i;
			if (levelBox.Value >= 1) {
				for(i=6;i<11;i++) {
					if(lA[i].Text != "") {
						lA[i].Visible = true;
						sA[i].Visible = true;
						cA[i*2].Visible = true;
						cA[(i*2)+1].Visible = true;
					}
					else {
						lA[i].Visible = false;
						sA[i].Visible = false;
						cA[i*2].Visible = false;
						cA[(i*2)+1].Visible = false;
						sA[i].Value = 0;
					}
				}
				label5.Visible = true;
				secBox.Visible = true;
			}
			else {
				for(i=6;i<11;i++) {
					lA[i].Visible = false;
					sA[i].Visible = false;
					cA[i*2].Visible = false;
					cA[(i*2)+1].Visible = false;
					sA[i].Value = 0;
				}
				label5.Visible = false;
				secBox.Visible = false;
			}
		}
	
		public MainForm()
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
				
			cClasses = new AllClasses(); 
			if(!cClasses.XMLLoad("gwc.xml")) {
				UpdateFiles();
				MessageBox.Show("Error loading XML file, Attempting to update.","Critical Error",MessageBoxButtons.OK,MessageBoxIcon.Error);
				if(!cClasses.XMLLoad("gwc.xml")) {
					MessageBox.Show("Could not download a satisfactory XML Data file. Please download updated files manually.","Critical Error",MessageBoxButtons.OK,MessageBoxIcon.Error);
				}
			}
			
			this.InitControls();
			
			this.primBox.Text = "Ranger";
			this.secBox.Text = "Warrior";
			
			this.maxAttributes = 5;
			this.attribBox.Text = this.maxAttributes.ToString();
			this.usedAttributes = 0;
			
		}
		
		//Simple copy file function that doesn't ask for write access to source file like .NET copier.
		public static void CopyFile(string s, string d) {
			FileStream destData = new FileStream("gwc.exe",FileMode.Create,FileAccess.Write);
			FileStream srcData = new FileStream("gwc.new.exe",FileMode.Open,FileAccess.Read);
			byte[] buf = new byte[256];
			int count = srcData.Read(buf,0,256);
			while (count > 0) {
				destData.Write(buf,0,count);
				count = srcData.Read(buf,0,256);
			}
			destData.Close();
			srcData.Close();
		}
		
		[STAThread]
		public static void Main(string[] args)
		{
			bool wait = true;
			if (args.GetLength(0) > 0) {
				if(args[0] == "update") {
					while(wait) {
						try {
							CopyFile("gwc.new.exe","gwc.exe");
							wait = false;
							System.Diagnostics.Process.Start("gwc.exe","finish");
						}
						catch {
						}
					}
				}
				else if (args[0] == "finish") {
					while(wait) {
						try {
							File.Delete("gwc.new.exe");
							wait = false;				
						}
						catch {
						}
					}
					Application.Run(new MainForm());
				}
			}
			else {
				Application.Run(new MainForm());	
			}
		}
		
		#region Big Ugly Code
		/// <summary>
		/// This method is required for Windows Forms designer support.
		/// Do not change the method contents inside the source code editor. The Forms designer might
		/// not be able to load this method if it was changed manually.
		/// </summary>
		private void InitializeComponent() {
			this.fileload = new System.Windows.Forms.OpenFileDialog();
			this.primBox = new System.Windows.Forms.ComboBox();
			this.levelbutton = new System.Windows.Forms.Button();
			this.label4 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.menuAbout = new System.Windows.Forms.MenuItem();
			this.menuFile = new System.Windows.Forms.MenuItem();
			this.menuUpdate = new System.Windows.Forms.MenuItem();
			this.nameBox = new System.Windows.Forms.TextBox();
			this.menuHelp = new System.Windows.Forms.MenuItem();
			this.menuLoad = new System.Windows.Forms.MenuItem();
			this.menuQuit = new System.Windows.Forms.MenuItem();
			this.linkLabel = new System.Windows.Forms.LinkLabel();
			this.menuLevel = new System.Windows.Forms.MenuItem();
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.secBox = new System.Windows.Forms.ComboBox();
			this.menuTools = new System.Windows.Forms.MenuItem();
			this.menuFree = new System.Windows.Forms.MenuItem();
			this.menuMode = new System.Windows.Forms.MenuItem();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.filesave = new System.Windows.Forms.SaveFileDialog();
			this.menuSave = new System.Windows.Forms.MenuItem();
			this.levelBox = new System.Windows.Forms.NumericUpDown();
			this.attribBox = new System.Windows.Forms.TextBox();
			this.menuHash = new System.Windows.Forms.MenuItem();
			this.label = new System.Windows.Forms.Label();
			((System.ComponentModel.ISupportInitialize)(this.levelBox)).BeginInit();
			this.SuspendLayout();
			// 
			// fileload
			// 
			this.fileload.DefaultExt = "*.txt";
			this.fileload.Filter = "XML Character|*.xml|Text Character|*.txt";
			// 
			// primBox
			// 
			this.primBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.primBox.Font = new System.Drawing.Font("Tahoma", 11.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.primBox.Location = new System.Drawing.Point(112, 96);
			this.primBox.Name = "primBox";
			this.primBox.Size = new System.Drawing.Size(128, 26);
			this.primBox.Sorted = true;
			this.primBox.TabIndex = 3;
			this.primBox.SelectedIndexChanged += new System.EventHandler(this.PrimBoxSelectedIndexChanged);
			// 
			// levelbutton
			// 
			this.levelbutton.Font = new System.Drawing.Font("Tahoma", 11.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.levelbutton.Location = new System.Drawing.Point(408, 56);
			this.levelbutton.Name = "levelbutton";
			this.levelbutton.Size = new System.Drawing.Size(104, 23);
			this.levelbutton.TabIndex = 55;
			this.levelbutton.Text = "Level Up!";
			this.levelbutton.Visible = false;
			this.levelbutton.Click += new System.EventHandler(this.LevelbuttonClick);
			// 
			// label4
			// 
			this.label4.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label4.Location = new System.Drawing.Point(8, 96);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(72, 23);
			this.label4.TabIndex = 53;
			this.label4.Text = "Primary:";
			// 
			// label5
			// 
			this.label5.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label5.Location = new System.Drawing.Point(280, 96);
			this.label5.Name = "label5";
			this.label5.TabIndex = 54;
			this.label5.Text = "Secondary:";
			// 
			// menuAbout
			// 
			this.menuAbout.Index = 0;
			this.menuAbout.Text = "About";
			this.menuAbout.Click += new System.EventHandler(this.menuAboutClick);
			// 
			// menuFile
			// 
			this.menuFile.Index = 0;
			this.menuFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.menuSave,
						this.menuLoad,
						this.menuQuit});
			this.menuFile.Text = "File";
			// 
			// menuUpdate
			// 
			this.menuUpdate.Index = 1;
			this.menuUpdate.Text = "Update Files";
			this.menuUpdate.Click += new System.EventHandler(this.menuUpdateClick);
			// 
			// nameBox
			// 
			this.nameBox.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.nameBox.Location = new System.Drawing.Point(80, 16);
			this.nameBox.Name = "nameBox";
			this.nameBox.Size = new System.Drawing.Size(240, 27);
			this.nameBox.TabIndex = 1;
			this.nameBox.Text = "";
			// 
			// menuHelp
			// 
			this.menuHelp.Index = 3;
			this.menuHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.menuAbout});
			this.menuHelp.Text = "Help";
			// 
			// menuLoad
			// 
			this.menuLoad.Index = 1;
			this.menuLoad.Text = "Load...";
			this.menuLoad.Click += new System.EventHandler(this.menuLoadClick);
			// 
			// menuQuit
			// 
			this.menuQuit.Index = 2;
			this.menuQuit.Text = "Quit";
			this.menuQuit.Click += new System.EventHandler(this.menuQuitClick);
			// 
			// linkLabel
			// 
			this.linkLabel.Font = new System.Drawing.Font("Tahoma", 11.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.linkLabel.Location = new System.Drawing.Point(328, 336);
			this.linkLabel.Name = "linkLabel";
			this.linkLabel.Size = new System.Drawing.Size(184, 23);
			this.linkLabel.TabIndex = 14;
			this.linkLabel.TabStop = true;
			this.linkLabel.Text = "Released under the GPL";
			this.linkLabel.TextAlign = System.Drawing.ContentAlignment.BottomRight;
			this.linkLabel.LinkClicked += new System.Windows.Forms.LinkLabelLinkClickedEventHandler(this.LinkLabelLinkClicked);
			// 
			// menuLevel
			// 
			this.menuLevel.Index = 1;
			this.menuLevel.Text = "Level-Up";
			this.menuLevel.Click += new System.EventHandler(this.menuLevelClick);
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.menuFile,
						this.menuTools,
						this.menuMode,
						this.menuHelp});
			// 
			// secBox
			// 
			this.secBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.secBox.Font = new System.Drawing.Font("Tahoma", 11.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.secBox.Location = new System.Drawing.Point(384, 96);
			this.secBox.Name = "secBox";
			this.secBox.Size = new System.Drawing.Size(128, 26);
			this.secBox.Sorted = true;
			this.secBox.TabIndex = 4;
			this.secBox.SelectedIndexChanged += new System.EventHandler(this.SecBoxSelectedIndexChanged);
			// 
			// menuTools
			// 
			this.menuTools.Index = 1;
			this.menuTools.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.menuHash,
						this.menuUpdate});
			this.menuTools.Text = "Tools";
			// 
			// menuFree
			// 
			this.menuFree.Checked = true;
			this.menuFree.Index = 0;
			this.menuFree.Text = "Free";
			this.menuFree.Click += new System.EventHandler(this.menuFreeClick);
			// 
			// menuMode
			// 
			this.menuMode.Index = 2;
			this.menuMode.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
						this.menuFree,
						this.menuLevel});
			this.menuMode.Text = "Mode";
			// 
			// label2
			// 
			this.label2.Font = new System.Drawing.Font("Tahoma", 14.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label2.Location = new System.Drawing.Point(400, 16);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(64, 23);
			this.label2.TabIndex = 51;
			this.label2.Text = "Level:";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// label3
			// 
			this.label3.BackColor = System.Drawing.SystemColors.Control;
			this.label3.Font = new System.Drawing.Font("Tahoma", 14.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label3.ForeColor = System.Drawing.SystemColors.ControlText;
			this.label3.ImageAlign = System.Drawing.ContentAlignment.MiddleRight;
			this.label3.Location = new System.Drawing.Point(8, 56);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(192, 23);
			this.label3.TabIndex = 52;
			this.label3.Text = "Attribute Points Left:";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// filesave
			// 
			this.filesave.DefaultExt = "xml";
			this.filesave.Filter = "Character File | *.xml";
			this.filesave.Title = "Save Character";
			// 
			// menuSave
			// 
			this.menuSave.Index = 0;
			this.menuSave.Text = "Save...";
			this.menuSave.Click += new System.EventHandler(this.menuSaveClick);
			// 
			// levelBox
			// 
			this.levelBox.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.levelBox.Location = new System.Drawing.Point(464, 16);
			this.levelBox.Maximum = new System.Decimal(new int[] {
						20,
						0,
						0,
						0});
			this.levelBox.Minimum = new System.Decimal(new int[] {
						1,
						0,
						0,
						0});
			this.levelBox.Name = "levelBox";
			this.levelBox.Size = new System.Drawing.Size(48, 27);
			this.levelBox.TabIndex = 2;
			this.levelBox.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			this.levelBox.Value = new System.Decimal(new int[] {
						1,
						0,
						0,
						0});
			this.levelBox.LostFocus += new System.EventHandler(this.LevelBoxValueChanged);
			this.levelBox.ValueChanged += new System.EventHandler(this.LevelBoxValueChanged);
			// 
			// attribBox
			// 
			this.attribBox.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.attribBox.Location = new System.Drawing.Point(240, 56);
			this.attribBox.Name = "attribBox";
			this.attribBox.ReadOnly = true;
			this.attribBox.Size = new System.Drawing.Size(80, 27);
			this.attribBox.TabIndex = 0;
			this.attribBox.TabStop = false;
			this.attribBox.Text = "0";
			this.attribBox.WordWrap = false;
			// 
			// menuHash
			// 
			this.menuHash.Index = 0;
			this.menuHash.Text = "Create Hashes";
			this.menuHash.Click += new System.EventHandler(this.menuHashClick);
			// 
			// label
			// 
			this.label.Font = new System.Drawing.Font("Tahoma", 14.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label.Location = new System.Drawing.Point(8, 16);
			this.label.Name = "label";
			this.label.Size = new System.Drawing.Size(64, 23);
			this.label.TabIndex = 0;
			this.label.Text = "Name: ";
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(520, 365);
			this.Controls.Add(this.levelbutton);
			this.Controls.Add(this.linkLabel);
			this.Controls.Add(this.attribBox);
			this.Controls.Add(this.secBox);
			this.Controls.Add(this.label5);
			this.Controls.Add(this.primBox);
			this.Controls.Add(this.levelBox);
			this.Controls.Add(this.label4);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.label);
			this.Controls.Add(this.nameBox);
			this.Menu = this.mainMenu;
			this.Name = "MainForm";
			this.Text = "Guild Wars Character Builder";
			((System.ComponentModel.ISupportInitialize)(this.levelBox)).EndInit();
			this.ResumeLayout(false);
		}
		#endregion
		
		//Event handler for level change.  Make sure it is legal and if not rollback.
		void LevelBoxValueChanged(object sender, System.EventArgs e)
		{
			if((int)this.levelBox.Value*POINTS_PER_LEVEL < this.usedAttributes)
			{
				this.levelBox.Value = (decimal)this.currentLevel;
			}
			else
			{
				this.maxAttributes = (int)this.levelBox.Value * POINTS_PER_LEVEL;
				int j = this.maxAttributes - this.usedAttributes;
				this.attribBox.Text = j.ToString();
				this.currentLevel = (int)this.levelBox.Value;
				CheckLevel();
			}
		}
		
		//Event handler for changing the primary class.  Update attribute lists and update list of possible secondary classes.
		void PrimBoxSelectedIndexChanged(object sender, System.EventArgs e)
		{
			string secText;
	
			for(int i=0;i<6;i++) {
				lA[i].Text = cClasses.getAName(primBox.Text,i);
				this.tips.SetToolTip(lA[i],cClasses.getDesc(primBox.Text,i));
				if(cClasses.isPrim(primBox.Text,i)) {
					lA[i].ForeColor = System.Drawing.Color.DarkRed;
				}
				else {
					lA[i].ForeColor = System.Drawing.SystemColors.ControlText;
				}
				
				if(cClasses.getAName(primBox.Text,i) == null) {
					lA[i].Visible = false;
					sA[i].Visible = false;
					cA[i*2].Visible = false;
					cA[(i*2)+1].Visible = false;
					sA[i].Value = 0;
				}
				else {
					lA[i].Visible = true;
					sA[i].Visible = true;
					cA[i*2].Visible = true;
					cA[(i*2)+1].Visible = true;
				}
			}
			
			secText = this.secBox.Text;
			this.secBox.Items.Clear();
			this.secBox.Items.AddRange(cClasses.getNames());
			this.secBox.Items.Remove(this.primBox.Text);
			if(this.primBox.Text != secText) {
				this.secBox.Text = secText;
			}
			else {
				IEnumerator i = this.secBox.Items.GetEnumerator();
				i.MoveNext();
				this.secBox.Text = i.Current.ToString();
			}
		}
		
		//Event handler for secondary class change.  Update the attributes to reflect the new class.
		void SecBoxSelectedIndexChanged(object sender, System.EventArgs e)
		{
			int j=6;
			int i;
			for(i=1;i<6;i++) {
				if(cClasses.getAName(secBox.Text,i) == null || !cClasses.isPrim(secBox.Text,i)) {
					lA[j].Text = cClasses.getAName(secBox.Text,i);
					this.tips.SetToolTip(lA[j],cClasses.getDesc(secBox.Text,i));
					j++;
				}
			}
			for(i=0;i<11-j;i++) {
				lA[10-i].Text = "";
			}
			CheckLevel();

		}
		
		//Generic handler for all NumericUpDown controls for the attributes.  Make sure the change is legal and update the rest of the data if so.  If not roll back the changes.
		void SpinValueChanged(object sender, System.EventArgs e)
		{
			int i=(int) ((System.Windows.Forms.NumericUpDown)sender).Tag;
			if(RecalcAttrib())
			{
				this.previousValues[i] = Convert.ToInt32(sA[i].Value);
				this.UpdateDisplay();
			}
			else
			{
				sA[i].Value = this.previousValues[i];
			}
			
			if(sA[i].Value > 0 && sA[i].Value < 12) {
				cA[i*2].Text = this.stand[(int) sA[i].Value].ToString();
				cA[(i*2)+1].Text = this.stand[(int) sA[i].Value+1].ToString();
			}
			else if(sA[i].Value == 12) {
				cA[i*2].Text = this.stand[12].ToString();
				cA[(i*2)+1].Text = "--";
			}
			else {
				cA[i*2].Text = "--";
				cA[(i*2)+1].Text = this.stand[1].ToString();
			}
		}
		
		//Handler for the GPL link.  Spawn IE with the GPL web page.
		void LinkLabelLinkClicked(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e)
		{
			System.Diagnostics.Process.Start("IExplore","http://www.gnu.org/copyleft/gpl.html");

		}
		
		//Handler for the File->Quit.
		void menuQuitClick(object sender, System.EventArgs e) {
			this.Close();
		}
		
		//Handler for File->Save.  Saves an XML format character.  Uses the filesave dialog box to handle paths and directories.
		void menuSaveClick(object sender, System.EventArgs e) {
			string s;
			if(this.filesave.ShowDialog() == DialogResult.OK) {
				XmlTextWriter saveFile = new XmlTextWriter(this.filesave.FileName,System.Text.Encoding.ASCII);
				saveFile.WriteStartElement("GWCSAVE");
				saveFile.WriteStartElement("NAME");
				saveFile.WriteString(this.nameBox.Text);
				saveFile.WriteEndElement();
				saveFile.WriteStartElement("LEVEL");
				saveFile.WriteString(this.levelBox.Text);
				saveFile.WriteEndElement();
				saveFile.WriteStartElement("PCLASS");
				saveFile.WriteString(this.primBox.Text);
				saveFile.WriteEndElement();
				saveFile.WriteStartElement("SCLASS");
				saveFile.WriteString(this.secBox.Text);
				saveFile.WriteEndElement();
				saveFile.WriteStartElement("ATTRIBUTES");
				for(int i=0;i<9;i++) {
					if(sA[i].Value > 0) {
						s = lA[i].Text.Replace(' ','_');
						saveFile.WriteStartElement(s.TrimEnd(':'));
						saveFile.WriteString(sA[i].Value.ToString());
						saveFile.WriteEndElement();
					}
				}
				saveFile.WriteEndElement();
				saveFile.WriteEndElement();
				saveFile.Close();
			}
		}
		
		//Text character loaded.  Provides the ability to load old text-style character saves.
		bool LoadTextChar(string s) {
			try {
				int i;
				StreamReader loadFile = new StreamReader(s);
				for(i=0;i<9;i++) {
					sA[i].Value=0;
				}
				
				this.nameBox.Text = loadFile.ReadLine();
				this.levelBox.Text = loadFile.ReadLine();
				this.primBox.Text = loadFile.ReadLine();
				this.secBox.Text = loadFile.ReadLine();
				for(i=0;i<9;i++) {
					sA[i].Value = Convert.ToDecimal(loadFile.ReadLine());
				}
				loadFile.Close();
				return true;
			}
			catch {
				return false;
			}
		}
		
		//New XML character loader.  Should ignore any unknown elements, so future updates should not break older versions.
		bool LoadXmlChar(string s) {
			try {
				string prev="";
				int i;
				for(i=0;i<0;i++) sA[i].Value = 0;
				XmlTextReader xread = new XmlTextReader(s);
				while (xread.Read()) {
					 switch (xread.NodeType) {
					 	case XmlNodeType.Element:
					 		prev = xread.Name;
			            	break;
			 
			            case XmlNodeType.EndElement:
			             	break;
					 	
			            case XmlNodeType.Text:
			            	if(prev == "NAME") {
			            		this.nameBox.Text = xread.Value;
			            	} else if (prev == "PCLASS") {
			            		this.primBox.Text =  xread.Value;
			            	} else if (prev == "SCLASS") {
			            		this.secBox.Text =  xread.Value;
			            	} else if (prev == "LEVEL") {
			            		this.levelBox.Value = Convert.ToInt32(xread.Value);
			            	} else {
			            		for(i=0;i<9;i++) {
			            			if(lA[i].Text.TrimEnd(':') == prev.Replace('_',' ')) {
			            				sA[i].Value = Convert.ToInt32(xread.Value);
			            			}
			            		}
			            	}
			            	break;
			            default:
		             		break;
					}
				
				}
				return true;
			}
			catch {
				MessageBox.Show("Error loading XML file.","Critical Error",MessageBoxButtons.OK,MessageBoxIcon.Error);
				return false;
			}
		}
		
		//File->Load handler.  Distinguish between .txt and .xml saves.
		void menuLoadClick(object sender, System.EventArgs e) {
			if(this.fileload.ShowDialog() == DialogResult.OK) {
				if(this.fileload.FilterIndex == 2) {
					if(!LoadTextChar(this.fileload.FileName)) {	
						MessageBox.Show("Error loading character file.","Error",MessageBoxButtons.OK,MessageBoxIcon.Error);
					}
				}
				else {
					if(!LoadXmlChar(this.fileload.FileName)) {
						MessageBox.Show("Error loading character file.","Error",MessageBoxButtons.OK,MessageBoxIcon.Error);
					}
				}
			}
		}
		
		//Tools->Hash handler.  Create the md5 hash files.  Only really useful to create the server-side copies but exposed none-the-less.
		void menuHashClick(object sender, System.EventArgs e) {
			if(!File.Exists("gwc.xml")) {
				MessageBox.Show("Error! gwc.dat does not exist!  Update!","File Error",MessageBoxButtons.OK,MessageBoxIcon.Error);
			}
			if(!File.Exists("gwc.exe")) {
				MessageBox.Show("Error! gwc.exe does not exist!","File Error",MessageBoxButtons.OK,MessageBoxIcon.Error);
			}
			FileStream datFile = new FileStream("gwc.xml",FileMode.Open,FileAccess.Read);
			FileStream exeFile = new FileStream("gwc.exe",FileMode.Open,FileAccess.Read);
			FileStream datHash = new FileStream("gwc.xml.md5",FileMode.Create);
			FileStream exeHash = new FileStream("gwc.exe.md5",FileMode.Create);
			
			System.Security.Cryptography.MD5 hash = new System.Security.Cryptography.MD5CryptoServiceProvider();
			byte[] Data = hash.ComputeHash(datFile);
			datHash.Write(Data,0,Data.GetLength(0));
			datHash.Close();
			
			Data = hash.ComputeHash(exeFile);
			exeHash.Write(Data,0,Data.GetLength(0));
			exeHash.Close();
		}
		
		//Tools->Update handler.  Call the updatefiles function and then reload data sets.
		void menuUpdateClick(object sender, System.EventArgs e) {
			string s1,s2;
			s1=primBox.Text;
			s2=secBox.Text;
			if(UpdateFiles()) {
				cClasses.XMLLoad("gwc.xml");
				secBox.Items.Clear();
				secBox.Items.AddRange(cClasses.getNames());
				primBox.Items.Clear();
				primBox.Items.AddRange(cClasses.getNames());
				primBox.Text = s1;
				secBox.Text = s2;
			}
			
		}
		
		//Quick and dirty About box.
		void menuAboutClick(object sender, System.EventArgs e) {
			MessageBox.Show("Guild Wars Character Builder 1.03\n(c) Joshua Lambert and Adam Kovalik\nFor more information see Readme.","About...",MessageBoxButtons.OK,MessageBoxIcon.Information);
		}
		
		//Level up!
		void LevelbuttonClick(object sender, System.EventArgs e)
		{
			if (this.levelBox.Value < MAX_LEVEL) this.levelBox.Value++;
		}
		
		
		//Free mode, can level up and down as you choose.
		void menuFreeClick(object sender, System.EventArgs e) {
			menuFree.Checked = true;
			menuLevel.Checked = false;
			levelbutton.Visible = false;
			levelBox.ReadOnly = false;
		}
		
		//Enter Level-Up! mode, where you go level-by-level.
		void menuLevelClick(object sender, System.EventArgs e) {
			menuFree.Checked = false;
			menuLevel.Checked = true;
			levelbutton.Visible = true;
			levelBox.ReadOnly = true;
		}
	}
}
